@@ -36,7 +36,8 @@ extern gboolean
 mixgtk_init (int argc, char *argv[]);
 
 /* enter the main mixgtk loop */
-#define mixgtk_main gtk_main
+extern void
+mixgtk_main (void);
 
 /* clean up */
 extern void
