@@ -27,10 +27,6 @@
 
 #include <gtk/gtk.h>
 
-/* initialise the color selection dialog */
-extern gboolean
-mixgtk_colorsel_init (void);
-
 /* gtk callbacks */
 extern void
 on_colors_activate (GtkWidget *w, gpointer p);
