@@ -28,15 +28,22 @@
 #include <gtk/gtk.h>
 #include <mixlib/mix_vm.h>
 
+/* enumeration of gmixvm dialogs */
+typedef enum {
+  MIXGTK_MAIN,	          /* main app window */
+  MIXGTK_WORD_DIALOG,	  /* input dialog for a mix word */
+  MIXGTK_ABOUT_DIALOG,
+  MIXGTK_GOTO_DIALOG,
+  MIXGTK_COLOR_DIALOG,    /* color customization dialog */
+  MIXGTK_COLORSEL_DIALOG, /* color selection dialog */
+  MIXGTK_FONTSEL_DIALOG,  /* font selection dialog */
+  MIXGTK_DEVFORM_DIALOG,  /* device format config dialog */
+  MIXGTK_EXTERNPROG_DIALOG	/* external programs dialog */
+} mixgtk_dialog_id_t;
+
 /* enumeration of mixvm widget ids */
 typedef enum {
-  MIXGTK_WIDGET_MAIN,		/* main app window */
   MIXGTK_WIDGET_STATUSBAR,
-  MIXGTK_WIDGET_WORD_DIALOG,	/* input dialog for a mix word */
-  MIXGTK_WIDGET_ABOUT_DIALOG,
-  MIXGTK_WIDGET_GOTO_DIALOG,
-  MIXGTK_WIDGET_COLORSEL_DIALOG, /* color customization dialog */
-  MIXGTK_WIDGET_FONTSEL_DIALOG,	/* font selection dialog */
   MIXGTK_WIDGET_MIXVM,		/* virtual machine */
   MIXGTK_WIDGET_PROMPT,		/* command prompt */
   MIXGTK_WIDGET_LOG,		/* command output */
@@ -69,6 +76,9 @@ typedef enum {
 extern gboolean
 mixgtk_widget_factory_init (const char *glade_file);
 
+/* get a dialog */
+extern GtkWidget *
+mixgtk_widget_factory_get_dialog (mixgtk_dialog_id_t dlg);
 
 /* get a widget */
 extern GtkWidget *
@@ -76,8 +86,8 @@ mixgtk_widget_factory_get (mixgtk_widget_id_t widget);
 
 /* Get a widget by name */
 extern GtkWidget *
-mixgtk_widget_factory_get_by_name (const gchar *name);
-
+mixgtk_widget_factory_get_child_by_name (mixgtk_dialog_id_t dlg,
+					 const gchar *name);
 
 #endif /* MIXGTK_WIDGETS_H */
 
