@@ -39,6 +39,8 @@ typedef struct mix_vm_cmd_dispatcher_t mix_vm_cmd_dispatcher_t;
 typedef enum {
   MIX_CMD_HELP = 0,		/* echo help message */
   MIX_CMD_LOAD,			/* load a mix program */
+  MIX_CMD_EDIT,			/* edit mixal source */
+  MIX_CMD_COMPILE,		/* compile mixal source */
   MIX_CMD_RUN,			/* run a loaded program */
   MIX_CMD_NEXT,			/* run next instruction */
   MIX_CMD_LOC,			/* print location pointer */
@@ -97,6 +99,25 @@ mix_vm_cmd_dispatcher_new (FILE *out_fd, /* output messages file */
 extern void
 mix_vm_cmd_dispatcher_delete (mix_vm_cmd_dispatcher_t *dis);
 
+
+/* set editor and compiler templates */
+extern void
+mix_vm_cmd_dispatcher_set_editor (mix_vm_cmd_dispatcher_t *dis,
+				  const gchar *edit_tplt);
+
+extern void
+mix_vm_cmd_dispatcher_set_assembler (mix_vm_cmd_dispatcher_t *dis,
+				     const gchar *asm_tplt);
+
+extern const gchar *
+mix_vm_cmd_dispatcher_get_src_file_path (const mix_vm_cmd_dispatcher_t *dis);
+
+extern const gchar *
+mix_vm_cmd_dispatcher_get_editor (const mix_vm_cmd_dispatcher_t *dis);
+
+extern const gchar *
+mix_vm_cmd_dispatcher_get_assembler (const mix_vm_cmd_dispatcher_t *dis);
+
 /* dispatch a command */
 extern gboolean /* TRUE if success, FALSE otherwise */
 mix_vm_cmd_dispatcher_dispatch (mix_vm_cmd_dispatcher_t *dis,
